import Terrain from "./Terrain.js";
import FileLoader from "../../js/FileLoader.js";
import TilorswiftEvent from "./events/TilorswiftEvent.js";
import Mouse from "./Mouse.js";
import Tileset from "./Tileset.js";
import Tilorswift from "./Tilorswift.js";
import TilorswiftSavedEvent from "./events/TilorswiftSavedEvent.js";
import MainMenu from "./menu/MainMenu.js";
import MenuGroup from "./menu/MenuGroup.js";
import MainMenuEntry from "./menu/MainMenuEntry.js";
import TilorswiftMenuSaveClickedEvent from "./events/TilorswiftMenuSaveClickedEvent.js";
import TilesetPickerWidget from "./menu/TilesetPickerWidget.js";
import WidgetBar from "./menu/WidgetBar.js";
import EntrancePointWidget from "./menu/EntrancePointWidget.js";
import Brush from "./Brush.js";
import BrushMode from "./BrushMode.js";

let loader = new FileLoader('../levels/level.json');
let image = new Image();
image.src = '../graphics/tileset-landscape01.jpg';

image.onload = function () {
    let terrain = Terrain.createFromJson(loader.getContent());
    let map = document.getElementById('map');
    map.appendChild(terrain.getElement());

    let brush = new Brush();

    let tileset = new Tileset(image, 8, 3);

    let widgetBar = new WidgetBar('widget-bar');

    let tilesetPicker = new TilesetPickerWidget(tileset, brush);
    widgetBar.addWidget(tilesetPicker);

    let entrancePicker = new EntrancePointWidget(widgetBar, brush);
    widgetBar.addWidget(entrancePicker);
    document.body.appendChild(widgetBar.getElement());

    let mouse = new Mouse();

    let mainbar = new MainMenu('mainbar');

    let menuFile = new MenuGroup('Datei');
    menuFile.addMenuEntry(
        new MainMenuEntry('Speichern', TilorswiftMenuSaveClickedEvent)
    );

    mainbar.addMenuGroup(menuFile);

    document.body.appendChild(mainbar.getElement());

    window.addEventListener(
        TilorswiftEvent.FIELD_CLICKED,
        (event) => {
            if (brush.mode === BrushMode.TERRAIN && !event.getField().isEntrancePoint) {
                switch (event.button) {
                    case 0:
                        event.getField().setIndex(terrain.brushTileIndex);
                        break;
                    case 2:
                        event.getField().setIndex(-1);
                        break;
                }
            } else if (brush.mode === BrushMode.ENTRANCE) {
                if (event.getField().index === -1) {
                    let coordinates = terrain.getFieldCoordinates(event.getField());
                    terrain.setEntrancePoint(coordinates.x, coordinates.y);
                    brush.mode = BrushMode.TERRAIN;
                    widgetBar.enableWidgets();
                }
            }
        }
    );

    window.addEventListener(
        'contextmenu',
        (event) => {
            event.preventDefault();
        }
    );

    window.addEventListener(
        TilorswiftEvent.FIELD_ENTERED,
        (event) => {
            if (mouse.isPressedLeft) {
                event.getField().setIndex(terrain.brushTileIndex);
            } else if (mouse.isPressedRight) {
                event.getField().setIndex(-1);
            }
        }
    );

    /* Prevent Firefox's default drag and drop for images */
    document.addEventListener(
        'dragstart',
        function (event) {
            event.preventDefault();
        }
    );

    window.addEventListener(
        TilorswiftEvent.MENU_SAVE_CLICKED,
        function () {
            Tilorswift.saveTerrainToFile(terrain);
            window.dispatchEvent(new TilorswiftSavedEvent());
        }
    );
};
